#include <PalmOS.h>

#include "plugin.h"
#include "enum-colorselect.h"
#include "db.h"
#include "enum.h"
#include "grid.h"

void MyCallBack( void *plugData)
{
    FormPtr form, oldForm;
    DmResID  gridColorID = 0;
    MemHandle colorRescH;
    IndexedColorType *colors, color, colorsReturn[GridUILastColorTableEntry];
    UInt32 version;
    UInt32 winDepth;
    DmOpenRef appli;
    UInt16 colorRescIndex;
    UInt16 exit = 0;
    Err err;

    if (FtrGet(sysFtrCreator, sysFtrNumROMVersion, &version) == 0) {
#define ver(a,b) sysMakeROMVersion(a, b, 0, sysROMStageRelease, 0)
        if (version < ver(3,5)) {
#undef ver
            return;
        }
    }

    WinScreenMode(winScreenModeGetSupportedDepths, NULL, NULL, &winDepth, NULL);

    if ((winDepth & 0x8080)) {
        gridColorID = wlistID_GridColors16;
    } else if (winDepth & 0x08) {
        gridColorID = wlistID_GridColors4;
    } else 
        return;


    if(!(appli = DmOpenDatabaseByTypeCreator( 'appl', 'DBOS', dmModeReadWrite))) {
        FrmCustomAlert( alertID_Debug, "appli 0", "", "");
        return;
    }

    colorRescIndex = DmFindResource( appli, 'BLST', gridColorID, NULL);

    if ( colorRescIndex == -1) {
        Char buf[256];

        StrPrintF(buf, "gridColorID: %ld\nwinDepth: %ld", gridColorID, winDepth);
        FrmCustomAlert( alertID_Debug, buf, "", "");
        DmCloseDatabase( appli);
        return;
    }

    if ((err = DmDetachResource( appli, colorRescIndex, &colorRescH)) != errNone) {
        ErrAlert(err);
        DmCloseDatabase( appli);
        return;
    }

    colors = MemHandleLock(colorRescH) + 2 * sizeof(UInt16) ;
    MemMove( colorsReturn, colors, GridUILastColorTableEntry * sizeof(IndexedColorType));

    oldForm = FrmGetActiveForm();
    form = FrmInitForm(formID_ColorSelect);
    FrmSetActiveForm(form);
#ifdef CONFIG_HANDERA
    if (FeatureSetHandera) {
        VgaFormModify(form, vgaFormModify160To240);
        UtilCenterForm(form, false);
    }
#endif

    while (exit == 0) {
        switch (FrmDoDialog(form)) {
        case ctlID_ColorSelect_Foreground:
            color = colors[GridUICellForeground];
            if (UIPickColor( &color, NULL, 0, "Color Select", 0))
                colorsReturn[GridUICellForeground] = color;
        break;
        case ctlID_ColorSelect_Background1:
            color = colors[GridUICellBackground];
            if (UIPickColor( &color, NULL, 0, "Color Select", 0))
                colorsReturn[GridUICellBackground] = color;
        break;
        case ctlID_ColorSelect_Background2:
            color = colors[GridUICellBackground2];
            if (UIPickColor( &color, NULL, 0, "Color Select", 0))
                colorsReturn[GridUICellBackground2] = color;
        break;
        case ctlID_ColorSelect_Header:
            color = colors[GridUIHeaderBackground];
            if (UIPickColor( &color, NULL, 0, "Color Select", 0))
                colorsReturn[GridUIHeaderBackground] = color;
        break;
        case ctlID_ColorSelect_HeaderText:
            color = colors[GridUIHeaderText];
            if (UIPickColor( &color, NULL, 0, "Color Select", 0))
                colorsReturn[GridUIHeaderText] = color;
        break;
        default:
        case ctlID_ColorSelect_OkayButton:
            exit = ctlID_ColorSelect_OkayButton;
        break;
        case ctlID_ColorSelect_CancelButton:
            exit = ctlID_ColorSelect_CancelButton;
        break;
        }

    }
    FrmDeleteForm(form);
    FrmSetActiveForm(oldForm);

    MemHandleUnlock( colorRescH);

    if (exit == ctlID_ColorSelect_OkayButton) {
        colors = MemHandleLock(colorRescH);
        DmWrite(colors, 2 * sizeof(UInt16), &colorsReturn, 
            GridUILastColorTableEntry * sizeof(IndexedColorType) - 2 * sizeof(UInt16));
        MemHandleUnlock( colorRescH);
    }
    DmAttachResource( appli, colorRescH, 'BLST', gridColorID);
    DmCloseDatabase( appli);
}

void MainPlug(void *plugData)
{
    Plug_SetAboutAlert(plugData, alertID_AboutBox);
    Plug_SetAction(plugData, ActionRunInChooserSelector, MyCallBack);
}
