#include <PalmOS.h>

#include "plugin.h"
#include "PlugLib.h"
#include "datasource.h"

#define plugCloseEvent (firstUserEvent + 0x200)

static void _PlugEventLoop(void *pluginData, Plug_HandleEventFuncType handleEvent, void * data);

void Plug_SetDriver(void *pluginData, DataSourceDriver *driver)
{
    PlugLib_globals * pluglibData = (PlugLib_globals *)pluginData;
    pluglibData->driver = driver;
}

void Plug_SetAboutAlert(void *pluginData, UInt16 alertID)
{
    PlugLib_globals * pluglibData = (PlugLib_globals *)pluginData;
    pluglibData->aboutAlertID = alertID;
}

void Plug_SetAction(void *pluginData, Plug_Action action, Plug_ActionCallBack * function)
{
    PlugLib_globals * pluglibData = (PlugLib_globals *)pluginData;
    UInt16 i;

    for (i = 0; i < pluglibData->numActions; i++) {
        if (pluglibData->actions[i].actionID == action) {
            pluglibData->actions[i].function = function;
            return;
        }
    }

    /* there isn't function defined for this action
     * we create a new one
     */
    if (pluglibData->numActions < 10) {
        pluglibData->actions[pluglibData->numActions].function = function;
        pluglibData->actions[pluglibData->numActions].actionID = action;
        pluglibData->numActions++;
    }
}

DataSource * Plug_GetSource(void *pluginData)
{
    PlugLib_appliDataV1 * data = (PlugLib_appliDataV1 *) pluginData;
    return data->source;
}

UInt16 Plug_GetRecord(void *pluginData)
{
    PlugLib_appliDataV1 * data = (PlugLib_appliDataV1 *) pluginData;
    return data->record;
}

void Plug_ReturnToForm(void * pluginData, UInt16 formID)
{
    if (formID ==0) {
        EventType e;

        MemSet(&e, sizeof(e), 0);
        e.eType = plugCloseEvent;
        EvtAddEventToQueue(&e);
    }
}

void Plug_DoForm(void *pluginData, UInt16 formID, Plug_HandleEventFuncType handleEvent, void * data)
{
    FormPtr form, oldForm;
    EventType e;

    oldForm = FrmGetActiveForm();

    form = FrmInitForm(formID);
    FrmSetActiveForm(form);

    e.eType = frmOpenEvent;
    e.data.frmOpen.formID= formID;
    EvtAddEventToQueue(&e);    

    _PlugEventLoop(pluginData, handleEvent, data);

    FrmEraseForm(form);
    FrmDeleteForm(form);
    FrmSetActiveForm(oldForm);
}

void Plug_GotoForm(void *pluginData, UInt16 formID, Plug_HandleEventFuncType handleEvent, void * data)
{
}

void _PlugEventLoop(void *pluginData, Plug_HandleEventFuncType handleEvent, void * data)
{
    EventType event;

    do {
        EvtGetEvent(&event, evtWaitForever);
    
        if (!SysHandleEvent(&event)) {
            if (!handleEvent(&event, data))
                FrmDispatchEvent(&event);
        }
        
    } while (event.eType != plugCloseEvent);
}
