#ifndef _DB_H
#define _DB_H

#define CPU_TYPE CPU_68K
#include <PalmOS.h>
#ifndef CONFIG_HANDERA
#define displayExtentChangedEvent  ((eventsEnum)(firstUserEvent-2))
#endif

#include "leakdetect.h"
#include "datasource.h"
#include "section.h"

#define DBCreatorID 'DBOS'
#define DBTypeID 'DB99'
#define DBListTypeID 'LIST'
#define DBHelpTypeID 'HELP'
#define DBOverlayTypeID 'ovly'
#define DBPrefsVersion 0

#define TMPDatabase "DBOStmp"

#define offsetof(s,m) ( ((ULong) (&(s).m)) - ((ULong)&(s)) )

/* redefine the PalmOS 3.5 function TblGetItemPtr*/
#ifdef CONFIG_PALMOS31
#undef TblGetItemPtr
#define TblGetItemPtr(table, row, column)(table->items[row * table->numColumns + column].ptr)
#endif

#define updateInitForm          0x01
#define updateChangeForm        0x02
#define updateRedrawForm        0x04
#define updateChangeFont        0x08
#define updateResizeForm        0x10
#define updateChangeView        0x20
#define updateChangeRecords     0x40
/*   update codes from 0x1000 - 0x8000 are reserved for form specific use */


typedef struct {
    Char name[33];
    UInt16 fieldNum;
    UInt32 uniqID;
} LinkType;
typedef LinkType * LinkPtr;

extern DataSourceSchema *DesignViewInitialSchema;
extern DataSourceDriver *DesignViewDriver;
extern char *DesignViewName;
extern DataSource * CurrentSource;
extern UInt16 CurrentRecord;
extern Boolean DesignNewDB;
extern UInt16 TopVisibleRecord;
extern UInt16 CurrentCategory;
extern Boolean IsNewRecord;
extern privateRecordViewEnum PrivateRecordStatus;
extern DataSourceDriver * DriverList;
extern UInt16 ListViewEditor_ViewNum;
extern UInt16 LastFormId[];

/* Keep track of what features this PalmOS version has. */
typedef struct
{
    UInt32 OS30     :1;
    UInt32 OS31     :1;
    UInt32 OS32     :1;
    UInt32 OS35     :1;
    UInt32 OS40     :1;
    UInt32 HiRes    :1;
    UInt32 HiResP   :1;
    UInt32 Handera  :1;
    UInt32 Sony     :1;
} FeatureSetType;
extern FeatureSetType FeatureSet;

extern UInt32 WinDepth;
#ifdef CONFIG_SONY
extern UInt16 SonyLibRefNum;
extern UInt16 SonySilkLibRefNum;
#endif
#ifdef CONFIG_HANDERA
extern char * RotateString;
#endif

extern Boolean ChooserHandleEvent(EventPtr) SECT_HND;
extern Boolean DesignViewHandleEvent(EventPtr) SECT_HND;
extern Boolean ListViewHandleEvent(EventPtr) SECT_HND;
extern Boolean FindDialogHandleEvent(EventPtr) SECT_HND;
extern Boolean EditViewHandleEvent(EventPtr) SECT_HND;
extern Boolean ListViewEditorHandleEvent(EventPtr event) SECT_HND;
extern Boolean ChooserInfoDialogHandleEvent(EventPtr) SECT_HND;
extern Boolean DuplicateDlgHandleEvent(EventPtr event) SECT_HND;
extern Boolean CreateDlgHandleEvent(EventPtr event) SECT_HND;
extern Boolean RenameDlgHandleEvent(EventPtr event) SECT_HND;
extern Boolean SortEditor_HandleEvent(EventPtr event) SECT_UI2;
extern Boolean NamesEditorHandleEvent(EventType* event) SECT_UI2;
extern Boolean HandleCommonMenuEvent(UInt16 menuitemID) SECT_UI2;
extern Boolean NoteHandleEvent(EventPtr event) SECT_UI2;
extern Boolean FieldPropertiesHandleEvent(EventPtr event) SECT_UI2;
extern Boolean AppPrefsHandleEvent(EventPtr event) SECT_UI2;
extern Boolean LinkSelectHandleEvent( EventPtr event) SECT_UI2;
#ifdef CONFIG_ABOUT
extern Boolean AboutHandleEvent(EventPtr event) SECT_UI2;
#endif
#ifdef CONFIG_GLB_SCRIPTS
extern Boolean GlbScriptEditorHandleEvent( EventPtr event ) SECT_UI2;
#endif

extern void    DbgPrintF(const Char * fmt, ...) ;
extern char*   GetStringResource(UInt16 stringID);
extern void    PutStringResource(const char * str);
extern char *  CopyStringResource(UInt16 stringID);
extern void GlobalSearch(FindParamsPtr params);

/*this functions must be in the main section of the application*/
/*the global search load only the main section*/
extern Boolean IsNumber(const char * s);
extern Int32   String2Long(const char * cp);
extern Boolean MatchString(const char *, const char *, Boolean, Boolean);

typedef struct
{
    UInt16 sourceID;
    Char title[32];
    /* variable-sized key data follows the above fixed portion */
} ChooserRecord;

extern Boolean Chooser_Open(void);
extern void Chooser_Close(void);
extern Boolean Chooser_Seek(UInt16 * indexP, Int16 offset, Int16 direction);
extern DmOpenRef Chooser_Ref(void);
#if 0
extern void Chooser_AddDatabase(const char *title, UInt16 sourceID,
				void * key_data, UInt16 key_size) ;
#endif
extern void Chooser_RescanDatabases(void);
extern void Chooser_RenameDatabase(UInt16 sourceID,
				   void * old_key_data, UInt16 old_key_size,
				   void * new_key_data, UInt16 new_key_size,
				   const char* new_title) ;

extern Boolean CmpField(DataSource * source, DataSourceGetter* getter,
                void* rec_data, UInt16 i, const char* str,
                Boolean caseSens, Boolean wholeWord) SECT_UI2;

/*link.c functions*/
#ifdef CONFIG_LINK
extern void LinkGo( UInt32 chooserRecordID, UInt16 fieldNum, LinkPtr link) SECT_UI2;
extern Boolean LinkSelect( UInt32 chooserRecordID, UInt16 filedNum, LinkPtr link) SECT_UI2;
extern Boolean LinkUpdate( UInt32 chooserRecordID, UInt16 fieldNum, LinkPtr link) SECT_UI2;
extern Boolean LinkIsUsefull( UInt32 chooserRecordID) SECT_UI2;
extern DataSourceFieldType LinkFieldType( UInt32 chooserRecordID, UInt16 fieldNum) SECT_UI2;
#endif

/*duplicate.c function*/
extern void DuplicateDatabase(DataSource *src, DataSourceDriver * driver,
            const char* dst_title, Boolean everything) SECT_UI2;
extern DataSource * RebuildDatabase(DataSource * src, DataSourceSchema * schema,
            UInt32 * reorder) SECT_UI2;



#define FrmReturnToForm(x)  if ( (x) == 0 ) {  UInt16 lft = LastFormId[0]; \
        LastFormId[0] = LastFormId[1]; LastFormId[1] = lft; } \
  else {  LastFormId[0] = LastFormId[1];  LastFormId[1] = (x); } FrmReturnToForm(x);

#endif




