/*
 * DB: Note Field editor
 * Copyright (C) 2001-2003: Marc Chalain (marc-chalain@voila.fr)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <PalmOS.h>
#include <Window.h>

#include "db.h"
#ifdef CONFIG_HANDERA
#include <Vga.h>
#endif
#include "enum.h"

#include "util.h"
#include "note.h"


#define GetObjectPtr(f,i) FrmGetObjectPtr((f),FrmGetObjectIndex((f),(i)))

static void NoteUpdateScrollers() EDITSECT;
static void NoteScroll (WinDirectionType direction, UInt16 linesToScroll) EDITSECT;

UInt16 NotePopupForm( NotePtr note, Boolean editable)
{
	FormPtr noteForm, oldForm;
	FieldPtr fld;
	FieldAttrType attr;
	UInt16 retVal = noteUnchanged;
	ControlPtr ctl;

    if (!note) return retVal;

	oldForm = FrmGetActiveForm();
	noteForm = FrmInitForm(formID_Note);
	FrmSetActiveForm(noteForm);
	FrmSetEventHandler(noteForm, NoteHandleEvent);
    
    if (!(note->h)) {
        Char * s;

        note->h = AllocateMemHandle( 256);
        s = LockMemHandle(note->h);
        s[0] = '\0';
        UnlockMemHandle(note->h);
    }
	/**
	 * Initialize the form 
	 */
#ifdef CONFIG_HANDERA
	if (FeatureSet.Handera) {
		VgaFormModify(noteForm, vgaFormModify160To240);
		if (!UtilCheckSizeForm(noteForm, 0)) {
			FrmAlert(alertID_FormTooBig);
			goto done;
		}
	}
#endif
	fld = GetObjectPtr(noteForm, ctlID_Note_EditField);
	
    FldSetTextHandle(fld, note->h);
    NoteUpdateScrollers();
	
	if (note->title[0]) {
		FrmSetTitle (noteForm, note->title);
	} 
	
	FldSetMaxChars(fld, NOTE_MAXCHAR - 1);
	FldGetAttributes( fld, &attr);
	attr.editable = editable;
	if ( !editable ) {
		attr.underlined = noUnderline;
		FrmHideObject(noteForm, FrmGetObjectIndex(noteForm, ctlID_Note_QuickTitle));
	}
	attr.hasScrollBar = true;
	attr.autoShift = true;
	FldSetAttributes( fld, &attr);

	FrmDrawForm(noteForm);
	
	/* SW Dec 12
	  For some reason if we call FrmSetFocus on the EditField
	   here, we get a strange blinking cursor artifact when we
	   return to the edit screen.  So I'm leaving it out...
	*/
   //FrmSetFocus(noteForm, FrmGetObjectIndex(noteForm, ctlID_Note_EditField));

	
	/* Handle events until the button is pressed */
	FrmDoDialog(noteForm);

	if (editable) {

		ctl = GetObjectPtr(noteForm, ctlID_Note_QuickTitle);
		if (CtlGetValue(ctl)) {
			int i = 0;
			Char *fieldP = FldGetTextPtr(fld);
			while (i < NOTETITLE_MAXCHAR-1 && fieldP[i] != 0 && fieldP[i] != '\n')
				note->title[i] = fieldP[i++];
			note->title[i] = '\0';
		}


		if (FldDirty(fld))	{
			FldCompactText (fld);
			retVal = noteModified;
		}
	}
	FldSetTextHandle(fld, 0);

#if CONFIG_HANDERA
  done:	
#endif

	FrmDeleteForm(noteForm);
	FrmSetActiveForm(oldForm);
	FrmUpdateForm(FrmGetFormId(oldForm), updateRedrawForm);
    return retVal;
}


/***********************************************************************
 *
 * FUNCTION:    NoteUpdateScrollers
 *
 * DESCRIPTION: P13. Visually updates the scroll arrows.
 *
 * PARAMETERS:  frm - poInt16er to the Edit View form.
 *
 * RETURNED:    nothing
 *
 ***********************************************************************/
static void NoteUpdateScrollers()
{
	FieldPtr fieldP;
	ScrollBarPtr scrollBarP;

	UInt16 nPosition, nMax;
	UInt16 textHeight, fieldHeight;
    FormPtr form = FrmGetActiveForm();

	fieldP = GetObjectPtr(form,ctlID_Note_EditField);
	scrollBarP = GetObjectPtr(form,ctlID_Note_ScrollBar);

	FldGetScrollValues(fieldP, &nPosition, &textHeight, &fieldHeight);

	if (textHeight > fieldHeight){
		nMax = textHeight - fieldHeight + FldGetNumberOfBlankLines (fieldP);
	} else if (nPosition) {
		nMax = nPosition;
	} else {
		nMax = 0;
	}

	SclSetScrollBar(scrollBarP, nPosition, 0, nMax, fieldHeight - 1);


}



/***********************************************************************
 *
 * FUNCTION:    NoteScroll
 *
 * DESCRIPTION: P13. Scrolls the edit view a page or a line at a time.
 *
 * PARAMETERS:  direction - up or dowm
 *              oneLine   - true scroll one line, false scrolls one page
 *
 * RETURNED:    nothing
 *
 ***********************************************************************/
static void NoteScroll (WinDirectionType direction, UInt16 linesToScroll)
{
	FieldPtr fld;
	Int16 blankLines;

	fld = GetObjectPtr(FrmGetActiveForm(),ctlID_Note_EditField);
	if (FldScrollable (fld, direction))
	{
		blankLines = FldGetNumberOfBlankLines(fld);

		FldScrollField(fld, linesToScroll, direction);
		NoteUpdateScrollers();
	}
}



/***********************************************************************
 *
 * FUNCTION:		NoteHandleEvent
 *
 * DESCRIPTION:	P3. Handles processing of events for the edit form.
 *
 * PARAMETERS:		event	- the most recent event.
 *
 * RETURNED:		True if the event is handled, false otherwise.
 *
 ***********************************************************************/
Boolean NoteHandleEvent(EventPtr event)
{
	FormPtr		frm;
	FieldPtr	fld;
	Boolean	handled = false;


	switch (event->eType)
	{

// No need for open/close events. This form is entered with DoDialog

   	case keyDownEvent:
   		if (event->data.keyDown.chr == pageUpChr)
		{
			UInt16 fieldHeight, null1, null2;
			fld = GetObjectPtr(FrmGetActiveForm(),ctlID_Note_EditField);
			FldGetScrollValues(fld, &null1, &null2, &fieldHeight);
			NoteScroll (winUp, fieldHeight);
			handled = true;
		}
	
		else if (event->data.keyDown.chr == pageDownChr)
		{
			UInt16 fieldHeight, null1, null2;
			fld = GetObjectPtr(FrmGetActiveForm(),ctlID_Note_EditField);
			FldGetScrollValues(fld, &null1, &null2, &fieldHeight);
			NoteScroll (winDown, fieldHeight);
			handled = true;
		}
		else
		{
			frm = FrmGetActiveForm ();
			FrmHandleEvent (frm, event);
			NoteUpdateScrollers ();
			handled = true;
		}
		break;
			
	case sclRepeatEvent:
		if( (event->data.sclRepeat.newValue - event->data.sclRepeat.value) < 0)
			NoteScroll (winUp, event->data.sclRepeat.value - event->data.sclRepeat.newValue);
		else
	   		NoteScroll (winDown, event->data.sclRepeat.newValue - event->data.sclRepeat.value);
		break;
			
			
    case fldChangedEvent:
        NoteUpdateScrollers();
    break;

    case menuCmdBarOpenEvent:
        if (FeatureSet.OS35) {
            frm = FrmGetActiveForm();
            fld = GetObjectPtr(frm, ctlID_Note_EditField);
            if (fld)
                FldHandleEvent(fld, event);
            event->data.menuCmdBarOpen.preventFieldButtons = true;
        }
    return false;

    case menuEvent:
        MenuEraseStatus(0);

        handled = HandleCommonMenuEvent(event->data.menu.itemID);
    break;

    default:
    break;
    }
    return(handled);
}

#undef GetObjectPtr
