/*
 * DB: Popup list display
 * Copyright (C) 1990-2001: Tom Dyas (original code from differents other files)
 * Copyright (C) 2001-2003: Marc Chalain (marc-chalain@voila.fr)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <PalmOS.h>

#include "db.h"
#include "grid.h"
#include "popuplist.h"

#define GetObjectPtr(f,i) FrmGetObjectPtr((f),FrmGetObjectIndex((f),(i)))

static char ** Names;
static UInt16 * Actions;
static UInt16 Count;

void
SetupPopupList( FormPtr form, UInt16 lstID, ActionDetails *popup_menu)
{
	UInt16 i, j, lstIndex;
	RectangleType bounds;
	ListPtr lst;

    if (Count)
        FreePopupList(NULL);

	/* Count the number of Actions for the popup menu. */
	Count = 0;
	for (i = 0; popup_menu[i].nameID != 0; ++i) {
		if (popup_menu[i].needsFeatureSet30) {
			if (FeatureSet.OS30) Count++;
		} else
		Count++;
	}

	/* Allocate space for the strings and Actions. */
	Names = AllocateMemPtr(Count * sizeof(char *));
	Actions = AllocateMemPtr(Count * sizeof(UInt16));

	/* Fill in the name for each action. */
	for (i = j = 0; popup_menu[i].nameID != 0; ++i) {
		Boolean add = true;

		/* Do not add this item if a feature is missing. */
		if (popup_menu[i].needsFeatureSet30 && !FeatureSet.OS30)
			add = false;

		/* Add the item to the popup Actions. */
		if (add) {
			Names[j] = GetStringResource(popup_menu[i].nameID);
			Actions[j] = popup_menu[i].actionID;
			j++;
		}
	}

	/* Retrieve a pointer to the popup list. */
	lstIndex = FrmGetObjectIndex(form, lstID);
	lst = FrmGetObjectPtr(form, lstIndex);

	/* Setup the list choices, selection, and the trigger. */
	LstSetHeight(lst, Count);

	/* Determine the minimum width needed for the list. */
	FrmGetObjectBounds(form, lstIndex, &bounds);
	bounds.extent.x = 0;
	for (i = 0; i < Count; ++i) {
		UInt16 width = FntCharsWidth(Names[i], StrLen(Names[i])) + 10;
		if (width > bounds.extent.x) bounds.extent.x = width;
	}
 	FrmSetObjectBounds(form, lstIndex, &bounds);
}

void
FreePopupList( ActionDetails *popup_menu)
{
    UInt16 i;

    for (i = 0; i < Count; ++i) PutStringResource(Names[i]);
    DeallocateMemPtr(Names);
    DeallocateMemPtr(Actions);
    Count = 0;
}


UInt16 ShowPopupList( FormPtr form, UInt16 lstID, EventPtr event)
{
	ListPtr lst;
	RectangleType r;
	Int16 result;
	UInt16 action;

	lst = GetObjectPtr(form, lstID);

	switch( event->eType) {
	case tblSelectEvent:
		TblUnhighlightSelection(event->data.tblSelect.pTable);
		TblGetItemBounds(event->data.tblSelect.pTable,
				event->data.tblSelect.row,
				event->data.tblSelect.column, &r);
	break;
	case gridHeaderSelectEvent:
		((GridType*) event->data.tblSelect.pTable)->ops->GridGetHeaderBounds( (GridType*) event->data.tblSelect.pTable,
				event->data.tblSelect.column, &r);
	break;
	default:
	}

	LstSetListChoices(lst, Names, Count);
	LstSetPosition(lst, r.topLeft.x, r.topLeft.y);
	LstSetSelection(lst, noListSelection);
	result = LstPopupList(lst);
	if (result >= 0)
		action = Actions[(UInt16) result];
	else
		action = -1;

	LstSetListChoices(lst, NULL, 0);
	return action;
}
